#define INCL_DOSMISC
#define INCL_DOSPROCESS
#define INCL_DOSMODULEMGR
#define INCL_DOSERRORS

#include <os2.h>

#include "frame.h"
#include "povproto.h"  /* For Status_Info */
#include "povray.h"    /* For Stop_Flag */

#include "optin.h"     /* For parse_ini_file() */

//#include <stdlib.h>    /* For getenv() */
#include <string.h>

#include <signal.h>



#if PRECISION_TIMER_AVAILABLE
unsigned long hstart, hstop;
#endif





#ifdef UNDERFLOW
int matherr(x)
struct exception *x;
{
  switch (x->type)
  {
    case DOMAIN:
    case OVERFLOW:

      x->retval = 1.0e17;

      break;

    case SING:
    case UNDERFLOW:

      x->retval = 0.0;

      break;
  }

  return (1);
}

#endif



#if PRECISION_TIMER_AVAILABLE

void OS2_Timer_Start()
{
   DosQuerySysInfo(QSV_TIME_LOW, QSV_TIME_LOW, &hstart, sizeof(hstart));
}


void OS2_Timer_Stop()
{
   DosQuerySysInfo(QSV_TIME_LOW, QSV_TIME_LOW, &hstop, sizeof(hstop));
}

int OS2_Timer_Count()
{
  return ((hstop - hstart) * 100000 +
          (hstop - hstart + 5) / 10);
}

#endif /* PRECISION_TIMER_AVAILABLE */


void OS2_Abort_Handler(int signum)
{
  OS2_Abort_Start();

  if (signum > 0 && ++Stop_Flag < 5)
  {
    switch (signum)
    {
#ifdef SIGINT
      case SIGINT: Status_Info("\nGot %d SIGINT.", Stop_Flag); break;
#endif
#ifdef SIGPIPE
      case SIGPIPE: Status_Info("\nGot %d SIGPIPE.", Stop_Flag); break;
#endif
#ifdef SIGTERM
      case SIGTERM: Status_Info("\nGot %d SIGTERM.", Stop_Flag); break;
#endif
      default: Status_Info("\nGot %d unknown signal (%d).", Stop_Flag, signum); break;
    }
  }

  if (signum == 0 || Stop_Flag > 10)
  {
#ifdef SIGTERM
    signal(SIGTERM, SIG_DFL);
#endif
#ifdef SIGINT
    signal(SIGINT, SIG_DFL);
#endif
#ifdef SIGPIPE
    signal(SIGPIPE, SIG_DFL);
#endif
  }
}


void OS2_Abort_Start()
{
#ifdef SIGTERM
  signal(SIGTERM, OS2_Abort_Handler);
#endif
#ifdef SIGINT
  signal(SIGINT, OS2_Abort_Handler);
#endif
#ifdef SIGPIPE
  signal(SIGPIPE, OS2_Abort_Handler);
#endif
#if !defined(OVERFLOW) && defined(SIGFPE) /* avoid floating point exceptions */
  signal(SIGFPE, SIG_IGN);
#endif
}

/*****************************************************************************
*
* FUNCTION  UNIX_Process_Env
*
* INPUT
*
* OUTPUT
*
* RETURNS
*
* AUTHOR  Mike Fleetwood  Jan 1999
*
* DESCRIPTION  Process a default INI file pointed to by the environment
*              variable POVINI.  Leaves the global variable Option_String_Ptr
*              set to non-NULL if and only if the environment variable POVINI
*              points to a file which was read.
*
* CHANGES
*
******************************************************************************/

void OS2_Process_Env(void)
{
  if ((Option_String_Ptr = getenv("POVINI")) != NULL)
  {
    if (!parse_ini_file(Option_String_Ptr))
    {
      Warning(0.0, "Could not find '%s' as specified in POVINI environment.\n",
	      Option_String_Ptr);
      Option_String_Ptr = NULL;
    }
  }
  return;
}

/*****************************************************************************
*
* FUNCTION  UNIX_Process_Povray_Ini
*
* INPUT
*
* OUTPUT
*
* RETURNS
*
* AUTHOR  Mike Fleetwood  Jan 1999
*
* DESCRIPTION  If UNIX_Process_Env() processed a default INI file from the
*              POVINI environment variable then do nothing; otherwise search
*              for a default INI file, reading the first one found, from the
*              list:
*                  ./povray.ini
*                  $HOME/.povrayrc
*                  POV_LIB_DIR/povray.ini
*
* CHANGES
*
******************************************************************************/

void OS2_Process_Povray_Ini(char *s)
{
  char ini_name[FILE_NAME_MAX];
  TIB *pTib;
  PIB *pPib;
  PSZ pszDPathValue = "";

  if(DosGetInfoBlocks(&pTib, &pPib) == 0)
  {
     if(DosQueryModuleName(pPib->pib_hmte, sizeof(ini_name), ini_name) == 0)
     {
       char *p = ini_name;
       while(*p)
         p++;
       while(*p != '\\')
         p--;
       p++;
       strcpy(p, "povray.ini");
     }
   }

   if(Option_String_Ptr != NULL)
      return;

   if(parse_ini_file(ini_name))
      return;

   if(DosScanEnv("DPATH", &pszDPathValue) == NO_ERROR)
   {
      if(DosSearchPath(SEARCH_CUR_DIRECTORY | SEARCH_IGNORENETERRS, pszDPathValue, "povray.ini", ini_name, sizeof(ini_name)) == NO_ERROR)
      {
         if(parse_ini_file(ini_name))
            return;
      }
   }
   return;
}




#ifndef PM_POV

void OS2_init_povray(int argc, char **argv)
{
   DosSetPriority(PRTYS_PROCESS, PRTYC_IDLETIME, PRTYD_MINIMUM, 0);
}

#endif
